/*
*   printf.c
*   A C program demonstrating advanced conversions and  
*   formatting
*   Copyright (c) William H. Murray and Chris H. Pappas, 1998
*/

#include <stdio.h>

void main()
{
  char   c        =   'A',
         psz1[]   =   "In making a living today many no ",
         psz2[]   =   "longer leave any room for life.";
  int    iln      =   0,
         ivalue   =   1234;
  double dPi      =   3.14159265;

  /*          conversions            */

  /* print the c                     */
  printf("\n[%2d] %c",++iln,c);

  /* print the ASCII code for c      */
  printf("\n[%2d] %d",++iln,c);

  /* print character with ASCII 90   */
  printf("\n[%2d] %c",++iln,90);

  /* print ivalue as octal value     */
  printf("\n[%2d] %o",++iln,ivalue);

  /* print lower-case hexadecimal    */
  printf("\n[%2d] %x",++iln,ivalue);

  /* print upper-case hexadecimal    */
  printf("\n[%2d] %X",++iln,ivalue);



  /* conversions and format options  */

  /* minimum width 1                 */
  printf("\n[%2d] %c",++iln,c);

  /* minimum width 5, right-justify  */
  printf("\n[%2d] %5c",++iln,c);

  /* minimum width 5, left-justify   */
  printf("\n[%2d] %-5c",++iln,c);

  /* 33 non-null, automatically      */
  printf("\n[%d] %s",++iln,psz1);

  /* 31 non-null, automatically      */
  printf("\n[%d] %s",++iln,psz2);

  /* minimum 5 overridden, auto 33   */
  printf("\n[%d] %5s",++iln,psz1);

  /* minimum width 38, right-justify */
  printf("\n[%d] %38s",++iln,psz1);

  /* minimum width 38, left-justify  */
  printf("\n[%d] %-38s",++iln,psz2);

  /* default ivalue width, 4         */
  printf("\n[%d] %d",++iln,ivalue);

  /* printf ivalue with + sign       */
  printf("\n[%d] %+d",++iln,ivalue);

  /* minimum 3 overridden, auto 4    */
  printf("\n[%d] %3d",++iln,ivalue);

  /* minimum width 10, right-justify */
  printf("\n[%d] %10d",++iln,ivalue);

  /* minimum width 10, left-justify  */
  printf("\n[%d] %-d",++iln,ivalue);

  /* right justify with leading 0's  */
  printf("\n[%d] %010d",++iln,ivalue);

  /* using default number of digits  */
  printf("\n[%d] %f",++iln,dPi);

  /* minimum width 20, right-justify */
  printf("\n[%d] %20f",++iln,dPi);

  /* right-justify with leading 0's  */
  printf("\n[%d] %020f",++iln,dPi);

  /* minimum width 20, left-justify  */
  printf("\n[%d] %-20f",++iln,dPi);

  /* no longer available since R1.2  */
  /* left-justify with trailing 0's  */



  /* additional formatting precision */

  /* minimum width 19, print all 17  */
  printf("\n[%d] %19.19s",++iln,psz1);

  /* prints first 2 chars            */
  printf("\n[%d] %.2s",++iln,psz1);

  /* prints 2 chars, right-justify   */
  printf("\n[%d] %19.2s",++iln,psz1);

  /* prints 2 chars, left-justify    */
  printf("\n[%d] %-19.2s",++iln,psz1);

  /* using printf arguments          */
  printf("\n[%d] %*.*s",++iln,19,6,psz1);

  /* width 10, 8 to right of '.'     */
  printf("\n[%d] %10.8f",++iln,dPi);

  /* width 20, 2 to right-justify    */
  printf("\n[%d] %20.2f",++iln,dPi);

  /* 4 decimal places, left-justify  */
  printf("\n[%d] %-20.4f",++iln,dPi);

  /* 4 decimal places, right-justify */
  printf("\n[%d] %20.4f",++iln,dPi);

  /* width 20, scientific notation   */
  printf("\n[%d] %20.2e",++iln,dPi);

}

