//
//  advio.cpp
//  A C++ program demonstrating advanced conversions and
//  formatting member functions since Release 2.0. The program
//  will also demonstrate how to convert each of the older
//  Release 1.2 form statements.
//  Copyright (c) Chris H. Pappas and William H. Murray, 1998
//

#include <string.h>
#include <strstrea.h>

#define INULL_TERMINATOR 1

void row (void);
main( )
{
  char   c        =   'A',
         psz1[]   =   "In making a living today many no ", 
         psz2[]   =   "longer leave any room for life.";
  int    iln      =   0,
         ivalue   =   1234;
  double dPi      =   3.14159265;

  // new declarations needed for Release 2.0
  char psz_padstring5[5+INULL_TERMINATOR], 
  psz_padstring38[38+INULL_TERMINATOR];

  // conversions

  // print the c
  // R1.2 cout << form("\n[%2d] %c",++ln,c);
  // Notice that << has been overloaded to output char
  row( ); // [ 1]
  cout << c;

  // print the ASCII code for c
  // R1.2  form("\n[%2d] %d",++ln,c);
  row( ); // [ 2]
  cout << (int)c;

  // print character with ASCII 90
  // R1.2  form("\n[%2d] %c",++ln,90);
  row( ); // [ 3]
  cout << (char)90;

  // print ivalue as octal value
  // R1.2  form("\n[%2d] %o",++ln,ivalue);
  row( ); // [ 4]
  cout << oct << ivalue;

  // print lower-case hexadecimal
  // R1.2  form("\n[%2d] %x",++ln,ivalue);
  row( ); // [ 5]
  cout << hex << ivalue;

  // print upper-case hexadecimal
  // R1.2  form("\n[%2d] %X",++ln,ivalue);
  row( ); // [ 6] cout.setf(ios::uppercase);
  cout << hex << ivalue; 
  cout.unsetf(ios::uppercase);    // turn uppercase off 
  cout << dec;                    // return to decimal base

  // conversions and format options

  // minimum width 1
  // R1.2  form("\n[%2d] %c",++ln,c);
  row( ); // [ 7]
  cout << c;

  // minimum width 5, right-justify
  // R1.2  form("\n[%2d] %5c",++ln,c);
  row( ); // [ 8] 
  ostrstream(psz_padstring5,sizeof(psz_padstring5)) 
  << "    " << c << ends;
  cout << psz_padstring5;

  // minimum width 5, left-justify
  // R1.2  form("\n[%2d] %-5c",++ln,c);
  row( ); // [ 9] 
  ostrstream(psz_padstring5,sizeof(psz_padstring5)) 
  << c << "    " << ends;
  cout << psz_padstring5;

  // 33 automatically
  // R1.2  form("\n[%d] %s",++ln,psz1);
  row( ); // [10]
  cout << psz1;

  // 31 automatically
  // R1.2  form("\n[%d] %s",++ln,psz2);
  row( ); // [11]
  cout << psz2;

  // minimum 5 overriden, auto
  // R1.2  form("\n[%d] %5s",++ln,psz1);
  // notice that the width of 5 cannot be overridden!
  row( ); // [12]
  cout.write(psz1,5);

  // minimum width 38, right-justify
  // R1.2  form("\n[%d] %38s",++ln,psz1);
  // notice how the width of 38 ends with garbage data
  row( ); // [13]
  cout.write(psz1,38);

  // the following is the correct approach
  cout << "\n\nCorrected approach:\n"; 
  ostrstream(psz_padstring38,sizeof(psz_padstring38)) << "     " 
    << psz1 << ends;
  row( ); // [14]
  cout << psz_padstring38;

  // minimum width 38, left-justify
  // R1.2  form("\n[%d] %-38s",++ln,psz2); 
  ostrstream(psz_padstring38,sizeof(psz_padstring38)) 
    << psz2 << "       " << ends;
  row( ); // [15]
  cout << psz_padstring38;

  // default ivalue width
  // R1.2  form("\n[%d] %d",++ln,ivalue);
  row( ); // [16]
  cout << ivalue;

  // printf ivalue with + sign
  // R1.2  form("\n[%d] %+d",++ln,ivalue);
  row( ); // [17]
  cout.setf(ios::showpos);     // don't want row number with + 
  cout << ivalue;
  cout.unsetf(ios::showpos);

  // minimum 3 overridden, auto
  // R1.2  form("\n[%d] %3d",++ln,ivalue);
  row( ); // [18]
  cout.width(3); // don't want row number padded to width of 3 
  cout << ivalue;

  // minimum width 10, right-justify
  // R1.2  form("\n[%d] %10d",++ln,ivalue);
  row( ); // [19]
  cout.width(10);    // only in effect for first value printed 
  cout << ivalue;

  // minimum width 10, left-justify
  // R1.2  form("\n[%d] %-d",++ln,ivalue);
  row( ); // [20]
  cout.width(10);
  cout.setf(ios::left);
  cout << ivalue;
  cout.unsetf(ios::left);

  // right-justify with leading 0's
  // R1.2  form("\n[%d] %010d",++ln,ivalue); 
  row( ); // [21]
  cout.width(10);
  cout.fill('0');
  cout << ivalue;
  cout.fill(' ');

  // using default number of digits
  // R1.2  form("\n[%d] %f",++ln,dPi); 
  row( ); // [22]
  cout << dPi;

  // minimum width 20, right-justify 
  // R1.2  form("\n[%d] %20f",++ln,dPi);
  row( ); // [23]
  cout.width(20);
  cout << dPi;

  // right-justify with leading 0's
  // R1.2  form("\n[%d] %020f",++ln,dPi);
  row( ); // [24]
  cout.width(20);
  cout.fill('0');
  cout << dPi;
  cout.fill(' ');

  // minimum width 20, left-justify
  // R1.2  form("\n[%d] %-20f",++ln,dPi);
  row( ); // [25]
  cout.width(20);
  cout.setf(ios::left);
  cout << dPi;

  // left-justify with trailing 0's
  // R1.2  form("\n[%d] %-020f",++ln,dPi);
  row( ); // [26]
  cout.width(20);
  cout.fill('0');
  cout << dPi;
  cout.unsetf(ios::left);
  cout.fill(' ');

  // additional formatting precision

  // minimum width 19, print all 17
  // R1.2  form("\n[%d] %19.19s",++ln,psz1);
  row( ); // [27]
  cout << psz1;

  // prints first 2 chars
  // R1.2  form("\n[%d] %.2s",++ln,psz1);
  row( ); // [28]
  cout.write(psz1,2);

  // prints 2 chars, right-justify
  // R1.2  form("\n[%d] %19.2s",++ln,psz1);
  row( ); // [29]
  cout << "                  "; cout.write(psz1,2);

  // prints 2 chars, left-justify
  // R1.2  form("\n[%d] %-19.2s",++ln,psz1);
  row( ); // [30]
  cout.write(psz1,2);

  // using printf arguments
  // R1.2  form("\n[%d] %*.*s",++ln,19,6,psz1);
  row( ); // [31]
  cout << "              "; cout.write(psz1,6);

  // width 10, 8 to right of '.'
  // R1.2  form("\n[%d] %10.8f",++ln,dPi);
  row( ); // [32]
  cout.precision(9);
  cout << dPi;

  // width 20, 2 to right-justify
  // R1.2  form("\n[%d] %20.2f",++ln,dPi);
  row( ); // [33]
  cout.width(20);
  cout.precision(2);
  cout << dPi;

  // 4 decimal places, left-justify
  // R1.2  form("\n[%d] %-20.4f",++ln,dPi);
  row( ); // [34]
  cout.precision(4);
  cout << dPi;

  // 4 decimal places, right-justify
  // R1.2  form("\n[%d] %20.4f",++ln,dPi);
  row( ); // [35]
  cout.width(20);
  cout << dPi;

  // width 20, scientific notation
  // R1.2  form("\n[%d] %20.2e",++ln,dPi);
  row( ); // [36] cout.setf(ios::scientific); cout.width(20);
  cout << dPi; cout.unsetf(ios::scientific);

  return(0);
}

void row (void)
{
  static int ln=0;
  cout << "\n[";
  cout.width(2);
  cout << ++ln << "] ";
}

