//
//  nnr.cpp
//  C++ program illustrates object-oriented programming
//  with a linked list. This program keeps track of
//  employee data at Nineveh National Research (NNR).
//  Copyright (c) Chris H. Pappas and William H. Murray, 1997
//

#include <iostream.h>
#include <string.h>

// PARENT CLASS
class NNR {

friend class payroll_list;

protected:
  char lstname[20];
  char fstname[15];
  char job_title[30];
  char social_sec[12];
  double year_hired;
  NNR *pointer;
  NNR *next_link;

public:
  NNR(char *lname,char *fname,char *ss,
      char *job,double y_hired)
  {
    strcpy(lstname,lname);
    strcpy(fstname,fname);
    strcpy(social_sec,ss);
    strcpy(job_title,job);
    year_hired=y_hired;
    next_link=0;
  }

  NNR()
  {
    lstname[0]=NULL;
    fstname[0]=NULL;
    social_sec[0]=NULL;
    job_title[0]=NULL;
    year_hired=0;
    next_link=0;
  }

  void fill_lstname(char *l_name)
  {
    strcpy(lstname,l_name);
  }

  void fill_fstname(char *f_name)
  {
    strcpy(fstname,f_name);
  }

  void fill_social_sec(char *soc_sec)
  {
    strcpy(social_sec,soc_sec);
  }

  void fill_job_title(char *o_name)
  {
    strcpy(job_title,o_name);
  }

  void fill_year_hired(double y_hired)
  {
    year_hired=y_hired;
  }

  virtual void add_info() {
  }
  virtual void send_info()
  {
    cout << "\n\n" << lstname << ", " << fstname
      << "\n Social Security: #" << social_sec;
    cout << "\n Job Title: " << job_title;
    cout << "\n Year Hired: " << year_hired;
  }

};

//SUB OR DERIVED CHILD CLASS
class administration:public NNR {

friend class payroll_list;

private:
  double yearly_salary;

public:
  administration(char *lname,char *fname,char *ss,
                 char *job,double y_hired,
                 double y_salary):
                 NNR(lname,fname,ss,
                 job,y_hired)
  {
    yearly_salary=y_salary;
  }

  administration():NNR()
  {
    yearly_salary=0.0;
  }

  void fill_yearly_salary(double salary)
  {
    yearly_salary=salary;
  }

  void add_info()
  {
    pointer=this;
   }

  void send_info()
  {
    NNR::send_info();
    cout << "\n Yearly Salary: $" << yearly_salary;
  }

};


//SUB OR DERIVED CHILD CLASS
class salespersons:public NNR {

friend class payroll_list;

private:
  double disk_sales;
  double comm_rate;

public:
  salespersons(char *lname,char *fname,char *ss,
               char *job,double y_hired,
               double d_sales,double c_rate):
               NNR(lname,fname,ss,
               job,y_hired)
  {
    disk_sales=d_sales;
    comm_rate=c_rate;
  }

  salespersons():NNR()
  {
    disk_sales=0.0;
    comm_rate=0;
  }

  void fill_sales(double d_sales)
  {
    disk_sales=d_sales;
  }
  void fill_comm_rate(int c_rate)
  {
    comm_rate=c_rate;
  }

  void add_info()
  {
    pointer=this;
  }

  void send_info()
  {
    NNR::send_info();
    cout << "\n Sales (disks): " << disk_sales;
    cout << "\n Commission Rate: " << comm_rate;
  }

};


//SUB OR DERIVED CHILD CLASS
class technicians:public NNR {

friend class payroll_list;

private:
  double hourly_salary;

public:
  technicians(char *lname,char *fname,char *ss,char *job,
              double y_hired,double h_salary):
              NNR(lname,fname,ss,job,y_hired)
  {
    hourly_salary=h_salary;
  }

  technicians():NNR()
  {
    hourly_salary=0.0;
  }

  void fill_hourly_salary(double h_salary)
  {
    hourly_salary=h_salary;
  }

  void add_info()
  {
    pointer=this;
  }

  void send_info()
  {
    NNR::send_info();
    cout << "\n Hourly Salary: $" << hourly_salary;
  }

};


//SUB OR DERIVED CHILD CLASS
class supplies:public NNR {

friend class payroll_list;

private:
  double hourly_salary;

public:
  supplies(char *lname,char *fname,char *ss,char *job,
           double y_hired,double h_salary):
           NNR(lname,fname,ss,
           job,y_hired)
  {
    hourly_salary=h_salary;
  }

  supplies():NNR()
  {
    hourly_salary=0.0;
  }

  void fill_hourly_salary(double h_salary)
  {
    hourly_salary=h_salary;
  }

  void add_info()
  {
    pointer=this;
  }

  void send_info()
  {
    NNR::send_info();
    cout << "\n Hourly Salary: $" << hourly_salary;
  }

};


//FRIEND CLASS
class payroll_list {

private:
  NNR *location;

public:
  payroll_list()
  {
    location=0;
  }

  void print_payroll_list();

  void insert_employee(NNR *node);

  void remove_employee_id(char *social_sec);

};

void payroll_list::print_payroll_list()
{
  NNR *present=location;

  while(present!=0) {
    present->send_info();
    present=present->next_link;
  }
}

void payroll_list::insert_employee(NNR *node)
{
  NNR *current_node=location;
  NNR *previous_node=0;
  while (current_node != 0 &&
         strcmp(current_node->lstname,node->lstname) < 0) {
    previous_node=current_node;
    current_node=current_node->next_link;
  }
  node->add_info();
  node->pointer->next_link=current_node;
  if (previous_node==0)
    location=node->pointer;
  else
    previous_node->next_link=node->pointer;
}

void payroll_list::remove_employee_id(char *social_sec)
{
  NNR *current_node=location;
  NNR *previous_node=0;

  while(current_node != 0 &&
     strcmp(current_node->social_sec,social_sec) != 0) {
    previous_node=current_node;
    current_node=current_node->next_link;
  }

  if(current_node != 0 && previous_node == 0) {
    location=current_node->next_link;
    // delete current_node; needed if new() used in add_info()
  }
  else if(current_node != 0 && previous_node != 0) {
    previous_node->next_link=current_node->next_link;
    // delete current_node; needed if new() used in add_info()
  }
}

main()
{
  payroll_list workers;

  // static data to add to linked list
  salespersons salesperson1("Harddrive","Harriet","313-56-7884",
                            "Salesperson",1985,6.5,7.5);
  salespersons salesperson2("Flex","Frank","663-65-2312",
                            "Salesperson",1985,3.0,3.2);
  salespersons salesperson3("Ripoff","Randle","512-34-7612",
                            "Salesperson",1987,9.6,6.8);
  technicians techperson1("Align","Alice","174-43-6781",
                            "Technician",1989,12.55);
  technicians techperson2("Tightscrew","Tom","682-67-5312",
                            "Technician",1992,10.34);
  administration vice_president1("Stuckup","Stewart",
                            "238-18-1119","Vice President",
                             1980,40000.00);
  administration vice_president2("Learnedmore","Lawrence",
                            "987-99-9653","Vice President",
                             1984,45000.00);
  supplies supplyperson1("Allpart","Albert","443-89-3772",
                            "Supplies",1983,8.55);
  supplies supplyperson2("Ordermore","Ozel","111-44-5399",
                            "Supplies",1988,7.58);

  // add the nine workers to the linked list
  workers.insert_employee(&techperson1);
  workers.insert_employee(&vice_president1);
  workers.insert_employee(&salesperson1);
  workers.insert_employee(&supplyperson1);
  workers.insert_employee(&supplyperson2);
  workers.insert_employee(&salesperson2);
  workers.insert_employee(&techperson2);
  workers.insert_employee(&vice_president2);
  workers.insert_employee(&salesperson3);

  // print the linked list
  workers.print_payroll_list();

  // remove two workers from the linked list
  workers.remove_employee_id("238-18-1119");
  workers.remove_employee_id("512-34-7612");

  cout << "\n\n***********************************";

  // print the revised linked list
  workers.print_payroll_list();

  return (0);
}

