/*
*  PIE.C
*  A Pie Chart Application with Resources
*  Copyright (c) William H. Murray and Chris H. Pappas, 1998
*/

#include <windows.h>
#include <string.h>
#include <math.h>
#include "pie.h"

#define radius      180
#define maxnumwedge 10
#define pi          3.14159265359

HINSTANCE hInst;

LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);
BOOL CALLBACK AboutDlgProc(HWND,UINT,WPARAM,LPARAM);
BOOL CALLBACK PieDlgProc(HWND,UINT,WPARAM,LPARAM);

char szProgName[]="ProgName";
char szApplName[]="PieMenu";
char szCursorName[]="PieCursor";

char szTString[80]="(bar chart title area)";
unsigned int iWedgesize[maxnumwedge]={5,20,10,15};
long lColor[maxnumwedge]={0x0L,0xFFL,0xFF00L,0xFFFFL,0xFF0000L,
                          0xFF00FFL,0xFFFF00L,0xFFFFFFL,
                          0x8080L,0x808080L};

int WINAPI WinMain(HINSTANCE hInst,HINSTANCE hPreInst,
                   LPSTR lpszCmdLine,int nCmdShow)
{
  HWND hWnd;
  MSG  lpMsg;
  WNDCLASS wcApp;

  wcApp.lpszClassName=szProgName;
  wcApp.hInstance    =hInst;
  wcApp.lpfnWndProc  =WndProc;
  wcApp.hCursor      =LoadCursor(hInst,szCursorName);
  wcApp.hIcon        =LoadIcon(hInst,szProgName);
  wcApp.lpszMenuName =szApplName;
  wcApp.hbrBackground=(HBRUSH)GetStockObject(WHITE_BRUSH);
  wcApp.style        =CS_HREDRAW|CS_VREDRAW;
  wcApp.cbClsExtra   =0;
  wcApp.cbWndExtra   =0;
  if (!RegisterClass (&wcApp))
    return 0;

  hWnd=CreateWindow(szProgName,"C Pie Chart Program",
                    WS_OVERLAPPEDWINDOW,CW_USEDEFAULT,
                    CW_USEDEFAULT,CW_USEDEFAULT,
                    CW_USEDEFAULT,(HWND)NULL,(HMENU)NULL,
                    hInst,(LPSTR)NULL);
  ShowWindow(hWnd,nCmdShow);
  UpdateWindow(hWnd);
  while (GetMessage(&lpMsg,0,0,0))
  {
    TranslateMessage(&lpMsg);
    DispatchMessage(&lpMsg);
  }
  return(lpMsg.wParam);
}

BOOL CALLBACK AboutDlgProc(HWND hdlg,UINT messg,
                           WPARAM wParam,LPARAM lParam)
{
  switch (messg)
  {
    case WM_INITDIALOG:
      break;
    case WM_COMMAND:
      switch (wParam)
      {
        case IDOK:
          EndDialog(hdlg,TRUE);
          break;
        default:
          return FALSE;
      }
      break;
    default:
      return FALSE;
  }
  return TRUE;
}

BOOL CALLBACK PieDlgProc(HWND hdlg,UINT messg,
                         WPARAM wParam,LPARAM lParam)
{
  switch (messg)
  {
    case WM_INITDIALOG:
      return FALSE;
    case WM_COMMAND:
      switch (wParam)
      {
        case IDOK:
          GetDlgItemText(hdlg,DM_TITLE,szTString,80);
          iWedgesize[0]=GetDlgItemInt(hdlg,DM_P1,NULL,0);
          iWedgesize[1]=GetDlgItemInt(hdlg,DM_P2,NULL,0);
          iWedgesize[2]=GetDlgItemInt(hdlg,DM_P3,NULL,0);
          iWedgesize[3]=GetDlgItemInt(hdlg,DM_P4,NULL,0);
          iWedgesize[4]=GetDlgItemInt(hdlg,DM_P5,NULL,0);
          iWedgesize[5]=GetDlgItemInt(hdlg,DM_P6,NULL,0);
          iWedgesize[6]=GetDlgItemInt(hdlg,DM_P7,NULL,0);
          iWedgesize[7]=GetDlgItemInt(hdlg,DM_P8,NULL,0);
          iWedgesize[8]=GetDlgItemInt(hdlg,DM_P9,NULL,0);
          iWedgesize[9]=GetDlgItemInt(hdlg,DM_P10,NULL,0);
          EndDialog(hdlg,TRUE);
          break;
        case IDCANCEL:
          EndDialog(hdlg,FALSE);
          break;
        default:
          return FALSE;
      }
      break;
    default:
      return FALSE;
  }
  return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd,UINT messg,
                         WPARAM wParam,LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  HBRUSH hBrush;
  static short xClientView,yClientView;

  unsigned int iTotalWedge[maxnumwedge+1];
  int i,iNWedges;

  iNWedges=0;
  for (i=0;i<maxnumwedge;i++) {
    if(iWedgesize[i]!=0) iNWedges++;
  }

  iTotalWedge[0]=0;

  for (i=0;i<iNWedges;i++)
    iTotalWedge[i+1]=iTotalWedge[i]+iWedgesize[i];

  switch (messg)
  {
    case WM_SIZE:
      xClientView=LOWORD(lParam);
      yClientView=HIWORD(lParam);
      break;
    case WM_COMMAND:
      switch (wParam)
      {
        case IDM_ABOUT:
          DialogBox(hInst,"AboutDlgBox",hWnd,
                    (DLGPROC)AboutDlgProc);
          break;
        case IDM_INPUT:
          DialogBox(hInst,"PieDlgBox",hWnd,
			        (DLGPROC)PieDlgProc);
          InvalidateRect(hWnd,NULL,TRUE);
          UpdateWindow(hWnd);
          break;
        case IDM_EXIT:
          SendMessage(hWnd,WM_CLOSE,0,0L);
          break;
        default:
          break;
      }
      break;
    case WM_PAINT:
      hdc=BeginPaint(hWnd,&ps);

      SetMapMode(hdc,MM_ISOTROPIC);
      SetWindowExtEx(hdc,500,500,NULL);
      SetViewportExtEx(hdc,xClientView,-yClientView,NULL);
      SetViewportOrgEx(hdc,xClientView/2,yClientView/2,NULL);

      if (xClientView > 200)
        TextOut(hdc,strlen(szTString)*(-8/2),
                240,szTString,strlen(szTString));

      for(i=0;i<iNWedges;i++) {
        hBrush=CreateSolidBrush(lColor[i]);
        SelectObject(hdc,hBrush);
        Pie(hdc,-200,200,200,-200,
           (short)(radius*cos(2*pi*iTotalWedge[i]/
                   iTotalWedge[iNWedges])),
           (short)(radius*sin(2*pi*iTotalWedge[i]/
                   iTotalWedge[iNWedges])),
           (short)(radius*cos(2*pi*iTotalWedge[i+1]/
                   iTotalWedge[iNWedges])),
           (short)(radius*sin(2*pi*iTotalWedge[i+1]/
                   iTotalWedge[iNWedges])));
      }


      ValidateRect(hWnd,NULL);
      EndPaint(hWnd,&ps);
      break;
    case WM_DESTROY:
      PostQuitMessage(0);
      break;
    default:
      return(DefWindowProc(hWnd,messg,wParam,lParam));
  }
  return(0);
}
