//
//  barchart.cpp
//  A Presentation Quality Bar Chart Application
//  using the MFC Library.
//  Copyright (c) William H. Murray and Chris H. Pappas, 1998
//

#include <afxwin.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "barchartr.h"   // resource IDs
#include "barchart.h"

#define maxnumbar 10
char szTString[80]="(bar chart title area)";
char szXString[80]="x-axis label";
char szYString[80]="y-axis label";
int iBarSize[maxnumbar]={20,10,40,50};
int m_cxClient,m_cyClient;

CTheApp theApp;

CMainWnd::CMainWnd()
{
  Create((AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW,
        LoadCursor(NULL,IDC_CROSS),
        (HBRUSH) GetStockObject(WHITE_BRUSH),NULL)),
        "Bar Chart Application with the MFC",
        WS_OVERLAPPEDWINDOW,rectDefault,NULL,"BarMenu");
}

void CMainWnd::OnSize(UINT,int x,int y)
{
  m_cxClient=x;
  m_cyClient=y;
}

void CMainWnd::OnPaint()
{
  CPaintDC dc(this);
  static DWORD dwColor[10]={RGB(0,0,0),        //black
                            RGB(245,0,0),      //red
                            RGB(0,245,0),      //green
                            RGB(0,0,245),      //blue
                            RGB(245,245,0),    //yellow
                            RGB(245,0,245),    //magenta
                            RGB(0,245,245),    //cyan
                            RGB(0,80,80),      //blend 1
                            RGB(80,80,80),     //blend 2
                            RGB(245,245,245)}; //white
  
  CFont newfont;
  CFont* oldfont;
  CBrush newbrush;
  CBrush* oldbrush;
  int i,iNBars,iBarWidth,iBarMax;
  int ilenMaxLabel;
  int x1,x2,y1,y2;
  int iBarSizeScaled[maxnumbar];
  char sbuffer[10],*strptr;

  iNBars=0;
  for (i=0;i<maxnumbar;i++) {
    if(iBarSize[i]!=0) iNBars++;
  }

  iBarWidth=400/iNBars;

  // Find bar with maximum height and scale
  iBarMax=iBarSize[0];
  for(i=0;i<iNBars;i++)
    if (iBarMax<iBarSize[i]) iBarMax=iBarSize[i];

  // Convert maximum y value to a string
  strptr=_itoa(iBarMax,sbuffer,10);
  ilenMaxLabel=strlen(sbuffer);
 
  // Scale bars in array.  Highest bar = 270
  for (i=0;i<iNBars;i++)
    iBarSizeScaled[i]=iBarSize[i]*(270/iBarMax);

  // Create custom viewport and map mode
  dc.SetMapMode(MM_ISOTROPIC);
  dc.SetWindowExt(640,400);
  dc.SetViewportExt(m_cxClient,m_cyClient);
  dc.SetViewportOrg(0,0);

  // Draw text to window if large enough
  if (m_cxClient > 200) {
  newfont.CreateFont(12,12,0,0,FW_BOLD, 
                     FALSE,FALSE,FALSE,OEM_CHARSET,
                     OUT_DEFAULT_PRECIS,
                     CLIP_DEFAULT_PRECIS,
                     DEFAULT_QUALITY,
                     VARIABLE_PITCH|FF_ROMAN,
                     "Roman");
  oldfont=dc.SelectObject(&newfont);
  dc.TextOut((300-(strlen(szTString)*10/2)),
              15,szTString,strlen(szTString));
  dc.TextOut((300-(strlen(szXString)*10/2)),
              365,szXString,strlen(szXString));
  dc.TextOut((90-ilenMaxLabel*12),70,strptr,ilenMaxLabel);
  // delete font objects
  dc.SelectObject(oldfont);
  newfont.DeleteObject();

  newfont.CreateFont(12,12,900,900,FW_BOLD,
                     FALSE,FALSE,FALSE,
                     OEM_CHARSET,OUT_DEFAULT_PRECIS,
                     CLIP_DEFAULT_PRECIS,
                     DEFAULT_QUALITY,
                     VARIABLE_PITCH|FF_ROMAN,
                     "Roman");
  oldfont=dc.SelectObject(&newfont);
  dc.TextOut(50,200+(strlen(szXString)*10/2),
             szYString,strlen(szYString));
  // delete font objects
  dc.SelectObject(oldfont);
  newfont.DeleteObject();
  }

  // Draw coordinate axis
  dc.MoveTo(99,49);
  dc.LineTo(99,350);
  dc.LineTo(500,350);
  dc.MoveTo(99,350);
 
  // Initial values
  x1=100;
  y1=350;
  x2=x1+iBarWidth;

  // Draw Each Bar
  for(i=0;i<iNBars;i++) {
    newbrush.CreateSolidBrush(dwColor[i]);
    oldbrush=dc.SelectObject(&newbrush);
    y2=350-iBarSizeScaled[i];
    dc.Rectangle(x1,y1,x2,y2);
    x1=x2;
    x2+=iBarWidth;
    // delete brush objects
    dc.SelectObject(oldbrush);
    newbrush.DeleteObject();
  }
}

int CMainWnd::OnCreate(LPCREATESTRUCT)
{
  UpdateWindow();
  return (0);
}

void CMainWnd::OnAbout()
{
  CDialog about("AboutDlgBox",this);
  about.DoModal();
}

void CBarDataDialog::OnOK()
{
  GetDlgItemText(DM_TITLE,szTString,80);
  GetDlgItemText(DM_XLABEL,szXString,80);
  GetDlgItemText(DM_YLABEL,szYString,80);
  iBarSize[0]=GetDlgItemInt(DM_P1,NULL,0);
  iBarSize[1]=GetDlgItemInt(DM_P2,NULL,0);
  iBarSize[2]=GetDlgItemInt(DM_P3,NULL,0);
  iBarSize[3]=GetDlgItemInt(DM_P4,NULL,0);
  iBarSize[4]=GetDlgItemInt(DM_P5,NULL,0);
  iBarSize[5]=GetDlgItemInt(DM_P6,NULL,0);
  iBarSize[6]=GetDlgItemInt(DM_P7,NULL,0);
  iBarSize[7]=GetDlgItemInt(DM_P8,NULL,0);
  iBarSize[8]=GetDlgItemInt(DM_P9,NULL,0);
  iBarSize[9]=GetDlgItemInt(DM_P10,NULL,0);
  CDialog::OnOK();
}

void CMainWnd::OnBarData()
{
  CBarDataDialog dlgBarData(this);
  if (dlgBarData.DoModal()==IDOK) {
    InvalidateRect(NULL,TRUE);
    UpdateWindow();
  }
};

void CMainWnd::OnExit()
{
  DestroyWindow();
}

BEGIN_MESSAGE_MAP(CMainWnd,CFrameWnd)
  ON_WM_PAINT()
  ON_WM_SIZE()
  ON_WM_CREATE()
  ON_COMMAND(IDM_ABOUT,OnAbout)
  ON_COMMAND(IDM_INPUT,OnBarData)
  ON_COMMAND(IDM_EXIT,OnExit)
END_MESSAGE_MAP()

BOOL CTheApp::InitInstance()
{
  m_pMainWnd=new CMainWnd();
  m_pMainWnd->ShowWindow(m_nCmdShow);
  m_pMainWnd->UpdateWindow();
  return TRUE;
}

