//
//  fourier.cpp
//  Drawing A Fourier Series with the use of
//  the MFC library.
//  Copyright (c) William H. Murray and Chris H. Pappas, 1998
//

#include <afxwin.h>
#include <string.h>
#include <math.h>
#include "fourierr.h"   // resource IDs
#include "fourier.h"

int m_cxClient,m_cyClient;
char mytitle[80]="Title";
int nterms=1;

CTheApp theApp;

CMainWnd::CMainWnd()
{
  Create((AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW,
         LoadCursor(NULL,IDC_CROSS),
         (HBRUSH) (GetStockObject(WHITE_BRUSH)),NULL)),
         "Fourier Series Application with the MFC",
         WS_OVERLAPPEDWINDOW,rectDefault,NULL,"FourierMenu");
}

void CMainWnd::OnSize(UINT,int x,int y)
{
  m_cxClient=x;
  m_cyClient=y;
}

void CMainWnd::OnPaint()
{
  CPaintDC dc(this);
  static DWORD dwColor[9]={RGB(0,0,0),        //black
                           RGB(245,0,0),      //red
                           RGB(0,245,0),      //green
                           RGB(0,0,245),      //blue
                           RGB(245,245,0),    //yellow
                           RGB(245,0,245),    //magenta
                           RGB(0,245,245),    //cyan
                           RGB(127,127,127),  //gray
                           RGB(245,245,245)}; //white

  int i,j,ltitle,ang;
  double y,yp;
  CBrush newbrush;
  CBrush* oldbrush;
  CPen newpen;
  CPen* oldpen;

  // create a custom drawing surface
  dc.SetMapMode(MM_ISOTROPIC);
  dc.SetWindowExt(500,500);
  dc.SetViewportExt(m_cxClient,-m_cyClient);
  dc.SetViewportOrg(m_cxClient/20,m_cyClient/2);

  ang=0;
  yp=0.0;

  newpen.CreatePen(BS_SOLID,2,RGB(0,0,0));
  oldpen=dc.SelectObject(&newpen);
   
  // draw x & y coordinate axes
  dc.MoveTo(0,240);
  dc.LineTo(0,-240);
  dc.MoveTo(0,0);
  dc.LineTo(400,0);
  dc.MoveTo(0,0);
  // draw actual Fourier waveform
  for (i=0; i<=400; i++) {
    for (j=1; j<=nterms; j++) {
      y=(150.0/((2.0*j)-1.0))*sin(((j*2.0)-1.0)*0.015708*ang);
      yp=yp+y;
    }
    dc.LineTo(i,(int) yp);
    yp-=yp;
    ang++;
  }

  // prepare to fill interior of waveform newbrush.
  newbrush.CreateSolidBrush(dwColor[7]);
  oldbrush=dc.SelectObject(&newbrush);
  dc.ExtFloodFill(150,10,dwColor[0],FLOODFILLBORDER);
  dc.ExtFloodFill(300,-10,dwColor[0],FLOODFILLBORDER);

  // print waveform title
  ltitle=strlen(mytitle); 
  dc.TextOut(200-(ltitle*8/2),185,mytitle,ltitle);

  // delete brush objects
  dc.SelectObject(oldbrush);
  newbrush.DeleteObject();
}

int CMainWnd::OnCreate(LPCREATESTRUCT)
{
  UpdateWindow();
  return (0);
}

void CMainWnd::OnAbout()
{
  CDialog about("AboutBox",this);
  about.DoModal();
}

void CFourierDataDialog::OnOK()
{
  GetDlgItemText(IDD_TITLE,mytitle,80);
  nterms=GetDlgItemInt(IDD_TERMS,NULL,0);
  CDialog::OnOK();
}

void CMainWnd::OnFourierData()
{
  CFourierDataDialog dlgFourierData(this);
  if (dlgFourierData.DoModal()==IDOK) {
    InvalidateRect(NULL,TRUE);
    UpdateWindow();
  }
};

void CMainWnd::OnExit()
{
  DestroyWindow();
}

BEGIN_MESSAGE_MAP(CMainWnd,CFrameWnd)
  ON_WM_PAINT()
  ON_WM_SIZE()
  ON_WM_CREATE()
  ON_COMMAND(IDM_ABOUT,OnAbout)
  ON_COMMAND(IDM_FOUR,OnFourierData)
  ON_COMMAND(IDM_EXIT,OnExit)
END_MESSAGE_MAP()

BOOL CTheApp::InitInstance()
{
  m_pMainWnd=new CMainWnd();
  m_pMainWnd->ShowWindow(m_nCmdShow);
  m_pMainWnd->UpdateWindow();
  return TRUE;
}

