//
//  gdi.cpp
//  An extension of the mfcswp.cpp application
//  that allows experimentation with graphics
//  drawing primitives.
//  Copyright (c) William H. Murray and Chris H. Pappas, 1998
//

#include <afxwin.h>
#include "gdi.h"

CgdiAApp theApp;

CMainWnd::CMainWnd()
{
  Create(NULL,"Experimenting With GDI Primitives",
         WS_OVERLAPPEDWINDOW,rectDefault,NULL,NULL);
}

void CMainWnd::OnPaint()
{
  static DWORD dwColor[9]={RGB(0,0,0),        //black
                           RGB(255,0,0),      //red
                           RGB(0,255,0),      //green
                           RGB(0,0,255),      //blue
                           RGB(255,255,0),    //yellow
                           RGB(255,0,255),    //magenta
                           RGB(0,255,255),    //cyan
                           RGB(127,127,127),  //gray
                           RGB(255,255,255)}; //white
  short xcoord;
  POINT polylpts[4],polygpts[5];

  CBrush newbrush;
  CBrush* oldbrush;
  CPen  newpen;
  CPen* oldpen;

  CPaintDC dc(this);
  // draws a wide black diagonal line
  newpen.CreatePen(PS_SOLID,6,dwColor[0]);
  oldpen=dc.SelectObject(&newpen);
  dc.MoveTo(0,0);
  dc.LineTo(640,430);
  dc.TextOut(70,20,"<-diagonal line",15);
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();
  
  // draws a blue arc
  newpen.CreatePen(PS_DASH,1,dwColor[3]);
  oldpen=dc.SelectObject(&newpen);
  dc.Arc(100,100,200,200,150,175,175,150);
  dc.TextOut(80,180,"small arc->",11);
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();

  // draws a wide green chord
  newpen.CreatePen(PS_SOLID,8,dwColor[2]);
  oldpen=dc.SelectObject(&newpen);
  dc.Chord(550,20,630,80,555,25,625,70);
  dc.TextOut(485,30,"chord->",7);
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();

  // draws and fills a red ellipse
  newpen.CreatePen(PS_SOLID,1,dwColor[1]);
  oldpen=dc.SelectObject(&newpen);
  newbrush.CreateSolidBrush(dwColor[1]);
  oldbrush=dc.SelectObject(&newbrush);
  dc.Ellipse(180,180,285,260);
  dc.TextOut(210,215,"ellipse",7);
  // delete brush objects
  dc.SelectObject(oldbrush);
  newbrush.DeleteObject();
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();

  // draws and fills a blue circle with ellipse function
  newpen.CreatePen(PS_SOLID,1,dwColor[3]);
  oldpen=dc.SelectObject(&newpen);
  newbrush.CreateSolidBrush(dwColor[3]);
  oldbrush=dc.SelectObject(&newbrush);
  dc.Ellipse(380,180,570,370);
  dc.TextOut(450,265,"circle",6);
  // delete brush objects
  dc.SelectObject(oldbrush);
  newbrush.DeleteObject();
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();

  // draws a black pie wedge and fills with green
  newpen.CreatePen(PS_SOLID,1,dwColor[0]);
  oldpen=dc.SelectObject(&newpen);
  newbrush.CreateSolidBrush(dwColor[2]);
  oldbrush=dc.SelectObject(&newbrush);
  dc.Pie(300,50,400,150,300,50,300,100);
  dc.TextOut(350,80,"<-pie wedge",11);
  // delete brush objects
  dc.SelectObject(oldbrush);
  newbrush.DeleteObject();
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();

  // draws a black rectangle and fills with gray
  newbrush.CreateSolidBrush(dwColor[7]);
  oldbrush=dc.SelectObject(&newbrush);
  dc.Rectangle(50,300,150,400);
  dc.TextOut(160,350,"<-rectangle",11);
  // delete brush objects
  dc.SelectObject(oldbrush);
  newbrush.DeleteObject();
    
  // draws a black rounded rectangle and fills with blue
  newbrush.CreateHatchBrush(HS_CROSS,dwColor[3]);
  oldbrush=dc.SelectObject(&newbrush);
  dc.RoundRect(60,310,110,350,20,20);
  dc.TextOut (120,310,"<------rounded rectangle",24);
  // delete brush objects
  dc.SelectObject(oldbrush);
  newbrush.DeleteObject();
  
  // draws several green pixels
  for(xcoord=400;xcoord<450;xcoord+=3)
    dc.SetPixel(xcoord,150,0L);
  dc.TextOut(455,145,"<-pixels",8);

  // draws several wide magenta lines with polyline
  newpen.CreatePen(PS_SOLID,3,dwColor[5]);
  oldpen=dc.SelectObject(&newpen);
  polylpts[0].x=10;
  polylpts[0].y=30;
  polylpts[1].x=10;
  polylpts[1].y=100;
  polylpts[2].x=50;
  polylpts[2].y=100;
  polylpts[3].x=10;
  polylpts[3].y=30;
  dc.Polyline(polylpts,4);
  dc.TextOut(10,110,"polyline",8);
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();

  // draws a wide cyan polygon and
  // fills with diagonal yellow
  newpen.CreatePen(PS_SOLID,4,dwColor[6]);
  oldpen=dc.SelectObject(&newpen);
  newbrush.CreateHatchBrush(HS_FDIAGONAL,dwColor[4]);
  oldbrush=dc.SelectObject(&newbrush);
  polygpts[0].x=40;
  polygpts[0].y=200;
  polygpts[1].x=100;
  polygpts[1].y=270;
  polygpts[2].x=80;
  polygpts[2].y=290;
  polygpts[3].x=20;
  polygpts[3].y=220;
  polygpts[4].x=40;
  polygpts[4].y=200;
  dc.Polygon(polygpts,5);
  dc.TextOut(70,210,"<-polygon",9);
  // delete brush objects
  dc.SelectObject(oldbrush);
  newbrush.DeleteObject();
  // delete pen objects
  dc.SelectObject(oldpen);
  newpen.DeleteObject();
}

BEGIN_MESSAGE_MAP(CMainWnd,CFrameWnd)
  ON_WM_PAINT()
END_MESSAGE_MAP()

BOOL CgdiAApp::InitInstance()
{
  m_pMainWnd=new CMainWnd();
  m_pMainWnd->ShowWindow(m_nCmdShow);
  m_pMainWnd->UpdateWindow();

  return TRUE;
}

